/*
 * Copyright (C) 2005-2007 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

/** \file recv.c the stand-alone audio stream receiver */

#include "para.h"

#include "list.h"
#include "sched.h"
#include "recv.h"
#include "recv.cmdline.h"
#include "fd.h"
#include "error.h"
#include "stdout.h"

/** the gengetopt args info struct */
struct recv_args_info conf;

/** always log to stderr */
INIT_STDERR_LOGGING(conf.loglevel_arg);

/** init array of error codes used by para_recv */
INIT_RECV_ERRLISTS;

static void *parse_config(int argc, char *argv[], int *receiver_num)
{
	int i;

	if (recv_cmdline_parser(argc, argv, &conf))
		return NULL;
	HANDLE_VERSION_FLAG("recv", conf);
	if (conf.list_receivers_given) {
		printf("available receivers: ");
		for (i = 0; receivers[i].name; i++)
			printf("%s%s", i? " " : "", receivers[i].name);
		printf("\nTry\n\tpara_recv -r '<receivername> -h'\n"
			"for help on <receivername>.\n");
		exit(EXIT_SUCCESS);
	}
	return check_receiver_arg(conf.receiver_arg, receiver_num);
}

static void rn_event_handler(struct task *t)
{
	struct receiver_node *rn = t->private_data;
	PARA_NOTICE_LOG("%s\n", PARA_STRERROR(-t->ret));
	rn->eof = 1;
	unregister_task(t);
}

/**
 * the main function of para_recv
 *
 * \param argc number of arguments
 * \param argv vector of arguments
 *
 * para_recv uses the specified receiver to receive an audio stream sent by
 * para_server. The received data is written to stdout.
 *
 * \return \a EXIT_SUCCESS on success, \a EXIT_FAILURE on errors.
 */
int main(int argc, char *argv[])
{
	int ret, r_opened = 0, receiver_num;
	struct receiver *r = NULL;
	struct receiver_node rn;
	struct stdout_task sot;
	struct sched s;

	s.default_timeout.tv_sec = 1;
	s.default_timeout.tv_usec = 0;

	memset(&rn, 0, sizeof(struct receiver_node));
	for (ret = 0; receivers[ret].name; ret++)
		receivers[ret].init(&receivers[ret]);
	ret = -E_RECV_SYNTAX;
	rn.conf = parse_config(argc, argv, &receiver_num);
	if (!rn.conf) {
		PARA_EMERG_LOG("%s", "parse failed\n");
		goto out;
	}
	r = &receivers[receiver_num];
	rn.receiver = r;
	ret = r->open(&rn);
	if (ret < 0)
		goto out;
	r_opened = 1;

	stdout_set_defaults(&sot);
	sot.buf = rn.buf;
	sot.loaded = &rn.loaded;
	sot.input_eof = &rn.eof;
	register_task(&sot.task);

	rn.task.private_data = &rn;
	rn.task.pre_select = r->pre_select;
	rn.task.post_select = r->post_select;
	rn.task.event_handler = rn_event_handler;
	sprintf(rn.task.status, "receiver node");
	register_task(&rn.task);

	ret = sched(&s);
out:
	if (r_opened)
		r->close(&rn);
	if (r)
		r->shutdown();
	if (ret < 0)
		PARA_ERROR_LOG("%s\n", PARA_STRERROR(-ret));
	return ret < 0? EXIT_FAILURE : EXIT_SUCCESS;
}
